﻿Option Strict On
Option Infer On

Imports SlimDX
Imports SlimDX.Direct2D
Imports SlimDX.DirectWrite
Imports D2DControlset

Friend Class Buildmenu
    Inherits GameControl
    Private m_selectedIndex As Integer

    Private entries As List(Of BuildmenuEntry)
    Private sound As SoundEffect

    Friend Event selectedItemChanged(sender As Object, e As EventArgs)

    Friend ReadOnly Property selectedItem As BuildmenuEntry
        Get
            If selectedIndex < 0 OrElse selectedIndex >= entries.Count Then Return Nothing
            Return entries.Item(selectedIndex)
        End Get
    End Property

    Private Property SelectedIndex As Integer
        Get
            Return m_selectedIndex
        End Get
        Set(value As Integer)
            m_selectedIndex = value
            If value >= 0 AndAlso Not Visible Then Visible = True
            If value < 0 AndAlso Visible Then Visible = False
            RaiseEvent selectedItemChanged(Me, EventArgs.Empty)
        End Set
    End Property

    Friend ReadOnly Property OptimalHeight As Integer
        Get
            Dim space = 10 * Game.getSizeFactor
            Dim entrySize = Width * 0.4F
            Dim border = (Width - entrySize) / 2.0F
            Dim textHeight = 30 * Game.getSizeFactor
            Dim entryHeight = entrySize + textHeight
            Return Convert.ToInt32(2 * border + (entries.Count - 1) * space + entries.Count * entryHeight)
        End Get
    End Property

    Protected Overrides Sub drawContent(e As D2DDrawEventArgs, inRect As RectangleF)
        Dim f = Game.getSizeFactor
        Dim entrySize = Width * 0.4F
        Dim textHeight = 30 * f
        Dim border = (Width - entrySize) / 2.0F
        Dim entryHeight = entrySize + textHeight
        Dim space = 10 * f
        Dim absLoc = AbsoluteLocation
        Dim format = e.TextFactory.CreateTextFormat("Segoe UI", FontWeight.Normal, FontStyle.Normal, FontStretch.Normal, 11 * f, Game.locale)
        format.TextAlignment = TextAlignment.Center
        format.ParagraphAlignment = ParagraphAlignment.Center
        For i = 0 To entries.Count - 1
            Dim imgRect As New RectangleF(absLoc.X + border, absLoc.Y + border + i * (entryHeight + space), entrySize, entrySize)
            Dim textRect As New RectangleF(absLoc.X, imgRect.Y + imgRect.Height, Width, textHeight)
            Dim entry = entries.Item(i)
            If entry.image IsNot Nothing Then e.Target.DrawBitmap(entry.image, imgRect)
            If i = SelectedIndex Then e.Target.DrawRectangle(ColorManager.GetBrush("defaultWhite"), imgRect, 2.5F * f)
            e.Target.DrawText(entry.name, format, textRect, ColorManager.GetBrush("defaultWhite"))
        Next
        format.Dispose()
    End Sub

    Sub New(ByVal entries As IEnumerable(Of BuildmenuEntry))
        Me.entries = New List(Of BuildmenuEntry)(entries)
        m_selectedIndex = -1
        sound = New SoundEffect("bt02.wma")
        AddHandler Game.window.KeyDown, AddressOf Window_KeyDown
    End Sub

    Private Sub Buildmenu_MouseClick(sender As Object, e As MouseEventArgs) Handles Me.MouseClick
        Dim f As Single = Game.getSizeFactor
        Dim entrySize As Single = Width * 0.4F
        Dim textHeight As Single = 30 * f
        Dim border As Single = (Width - entrySize) / 2.0F
        Dim entryHeight As Single = entrySize + textHeight
        Dim space As Single = (Height - (2 * border) - (entries.Count * entryHeight)) / (entries.Count - 1)
        Dim mouseRect As New RectangleF(e.X, e.Y, 1, 1)
        For i = 0 To entries.Count - 1
            Dim rect As New RectangleF(border, border + i * (entryHeight + space), entrySize, entrySize)
            If rect.IntersectsWith(mouseRect) Then SelectedIndex = i
        Next
    End Sub

    Private Sub Buildmenu_VisibleChanged(sender As Object, e As EventArgs) Handles Me.VisibleChanged
        If Not Visible AndAlso m_selectedIndex >= 0 Then SelectedIndex = -1
        If Visible AndAlso m_selectedIndex < 0 Then SelectedIndex = 0
    End Sub

    Private Sub Parent_Click(sender As Object, e As MouseEventArgs) Handles m_Parent.MouseClick
        If e.Button = MouseButtons.Right Then SelectedIndex = -1
    End Sub

    Private Sub Parent_MouseWheel(sender As Object, e As MouseEventArgs) Handles m_Parent.MouseWheel
        If SelectedIndex < 0 Then
            SelectedIndex = 0
        Else
            If e.Delta < 0 Then
                SelectedIndex = (SelectedIndex + 1) Mod entries.Count
            Else
                If SelectedIndex = 0 Then
                    SelectedIndex = entries.Count - 1
                Else
                    SelectedIndex = SelectedIndex - 1
                End If
            End If
        End If
        sound.play()
    End Sub

    Private Sub Window_KeyDown(sender As Object, e As KeyEventArgs)
        For i = 0 To Math.Min(entries.Count - 1, 8)
            If e.KeyCode = i + 49 Then
                If Not Visible Then Visible = True
                sound.play()
                SelectedIndex = i
                Exit For
            End If
        Next
        If e.KeyCode = Keys.D0 Then SelectedIndex = -1
        If e.KeyCode = Keys.B Then Visible = True
    End Sub
End Class

Friend MustInherit Class BuildmenuEntry
    Private m_Image As Bitmap
    Private m_Name As String
    Private m_Costs As Integer
    Private m_Radius As Double

    Friend ReadOnly Property image As Bitmap
        Get
            Return m_Image
        End Get
    End Property

    Friend ReadOnly Property name As String
        Get
            Return m_Name
        End Get
    End Property

    Friend ReadOnly Property costs As Integer
        Get
            Return m_Costs
        End Get
    End Property

    Friend ReadOnly Property radius As Double
        Get
            Return m_Radius
        End Get
    End Property

    Friend MustOverride Function getTower() As Tower

    Sub New(name As String, costs As Integer, image As Bitmap, radius As Double)
        m_Name = name
        m_Costs = costs
        m_Image = image
        m_Radius = radius
    End Sub
End Class

Friend Class BuildmenuEntry(Of T As {New, Tower})
    Inherits BuildmenuEntry

    Friend Overrides Function getTower() As Tower
        Return New T
    End Function

    Sub New(name As String, costs As Integer, image As Bitmap, radius As Double)
        MyBase.New(name, costs, image, radius)
    End Sub
End Class
