﻿Option Strict On
Option Infer On

Imports SlimDX
Imports SlimDX.Direct2D
Imports D2DControlset

Friend Class Lagometer
    Inherits D2DControl

    Private vals As List(Of Single)
    Private oldVals As List(Of Single)
    Private lastTime As DateTime

    Protected Overrides Sub OnDraw(e As D2DDrawEventArgs)
        If vals.Count >= Me.Width Then
            oldVals = New List(Of Single)(vals)
            vals.Clear()
        End If
        vals.Add(Convert.ToSingle((DateTime.Now - lastTime).TotalMilliseconds))
        lastTime = DateTime.Now
        Const max As Single = 105.0F
        Const lSize As Single = 1.0F
        Const optimalSpeed As Single = 1000.0F / 60.0F
        Const minimumSpeed As Single = 1000.0F / 30.0F
        Dim opacityChange As Double = 1 / Me.Width * 2
        Dim greenBrush As New SolidColorBrush(e.Target, New Color4(Color.Lime))
        Dim redBrush As New SolidColorBrush(e.Target, New Color4(Color.Red))
        Dim yellowBrush As New SolidColorBrush(e.Target, New Color4(Color.Yellow))
        Dim getX As Func(Of Integer, Single) = Function(i) (lSize * i) + (lSize / 2.0F) + AbsoluteLocation.X
        Dim getHeight As Func(Of Single, Single) = Function(h) Height * (h / max)
        For i = 0 To vals.Count - 1
            Dim b As SolidColorBrush = Nothing
            If vals.Item(i) <= optimalSpeed Then
                b = greenBrush
            ElseIf vals.Item(i) <= minimumSpeed Then
                b = yellowBrush
            Else
                b = redBrush
            End If
            b.Opacity = Convert.ToSingle(1 - (vals.Count - i - 1) * opacityChange)
            Dim lHeight As Single = getHeight(vals.Item(i))
            Dim lX As Single = getX(i)
            Dim p1 As New PointF(lX, Me.AbsoluteLocation.Y + Me.Height)
            Dim p2 As New PointF(lX, p1.Y - lHeight)
            e.Target.DrawLine(b, p1, p2, lSize)
        Next
        If oldVals IsNot Nothing Then
            For i = vals.Count - 1 To oldVals.Count - 1
                Dim b As SolidColorBrush = Nothing
                If oldVals.Item(i) <= optimalSpeed Then
                    b = greenBrush
                ElseIf oldVals.Item(i) <= minimumSpeed Then
                    b = yellowBrush
                Else
                    b = redBrush
                End If
                b.Opacity = Convert.ToSingle(1 - (oldVals.Count + vals.Count - i) * opacityChange)
                Dim lHeight As Single = getHeight(oldVals.Item(i))
                Dim lX As Single = getX(i)
                Dim p1 As New PointF(lX, Me.AbsoluteLocation.Y + Me.Height)
                Dim p2 As New PointF(lX, p1.Y - lHeight)
                e.Target.DrawLine(b, p1, p2, lSize)
            Next
        End If
        yellowBrush.Dispose()
        redBrush.Dispose()
        greenBrush.Dispose()
    End Sub

    Sub New()
        vals = New List(Of Single)
        lastTime = DateTime.Now
    End Sub

    Private Sub Largometer_VisibleChanged(sender As Object, e As EventArgs) Handles Me.VisibleChanged
        If Visible Then
            lastTime = DateTime.Now
            vals.Clear()
            oldVals = Nothing
        End If
    End Sub
End Class
