﻿Imports D2DControlset
Imports SlimDX
Imports SlimDX.Direct2D
Imports SlimDX.DirectWrite
Imports Helpers.Mathematics

Friend Class TDTrackBar
    Inherits D2DControl

    Private m_MinValue As Integer
    Private m_MaxValue As Integer
    Private m_Value As Integer
    Private m_Pixel As Integer
    Private m_ShowValue As Boolean
    Private sound1 As SoundEffect
    Private sound2 As SoundEffect
    Private m_MouseDownTime As DateTime
    Private m_Moved As Boolean

    ''' <summary>
    ''' Tritt ein, wenn mit der Maus gescrollt wurde.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event Scroll(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' Tritt ein, wenn sich der Wert der Scrollbar geändert hat.
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    ''' <remarks></remarks>
    Public Event ValueChanged(ByVal sender As Object, ByVal e As EventArgs)

    ''' <summary>
    ''' Der minimale Wert, den diese Scrollbar annehmen kann.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property MinValue As Integer
        Get
            Return m_MinValue
        End Get
        Set(ByVal value As Integer)
            m_MinValue = Math.Min(value, m_MaxValue)
            If m_Value < m_MinValue Then
                m_Value = m_MinValue
                RaiseEvent ValueChanged(Me, New EventArgs)
            End If
            SetPixel()
        End Set
    End Property

    ''' <summary>
    ''' Der maximale Wert, den diese Scollbar annehmen kann.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property MaxValue As Integer
        Get
            Return m_MaxValue
        End Get
        Set(ByVal value As Integer)
            m_MaxValue = Math.Max(value, m_MinValue)
            If m_Value > m_MaxValue Then
                m_Value = m_MaxValue
                RaiseEvent ValueChanged(Me, New EventArgs)
            End If
            SetPixel()
        End Set
    End Property

    ''' <summary>
    ''' Der Wert dieser Scollbar.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property Value As Integer
        Get
            Return m_Value
        End Get
        Set(ByVal value As Integer)
            m_Value = MathHelper.Clamp(value, m_MinValue, m_MaxValue)
            RaiseEvent ValueChanged(Me, New EventArgs)
            SetPixel()
        End Set
    End Property

    ''' <summary>
    ''' Gibt an, ob die Scrollbar ihren Wert anzeigen soll.
    ''' </summary>
    ''' <value></value>
    ''' <returns></returns>
    ''' <remarks></remarks>
    Public Property ShowValue As Boolean
        Get
            Return m_ShowValue
        End Get
        Set(ByVal value As Boolean)
            m_ShowValue = value
        End Set
    End Property

    Private ReadOnly Property trackWidth As Integer
        Get
            Dim rHeight As Single = Me.Height - (10.0F * Game.getSizeFactor)
            Return Convert.ToInt32(rHeight / 2 * 3)
        End Get
    End Property

    Private Sub SetPixel()
        Dim rWidth As Integer = Convert.ToInt32(Me.Width - (5.0F * Game.getSizeFactor))
        Dim AvailablePixel As Integer = rWidth - trackWidth - Convert.ToInt32(10.0F * Game.getSizeFactor)
        Dim ValueDifferenz As Integer = m_MaxValue - m_MinValue
        If ValueDifferenz = 0 Then
            m_Pixel = 0
            Exit Sub
        End If
        Dim ValuePercent As Double = GetProzentsatz(ValueDifferenz, m_Value - m_MinValue)
        m_Pixel = MathHelper.Clamp(CInt(GetProzentwert(AvailablePixel, ValuePercent)), 0, AvailablePixel)
    End Sub

    Protected Overrides Sub OnDraw(ByVal e As D2DDrawEventArgs)
        Dim b As Single = 2.5F * Game.getSizeFactor
        Dim dRect As New RectangleF(Me.AbsoluteLocation.X + b, Me.AbsoluteLocation.Y + b, Me.Width - 2 * b, Me.Height - 2 * b)
        drawDiamond(e, dRect)
        Dim tRect As New RectangleF(dRect.X + 5.0F * Game.getSizeFactor + m_Pixel, dRect.Y + 3.5F * Game.getSizeFactor, trackWidth, dRect.Height - 7.0F * Game.getSizeFactor)
        fillDiamond(e, tRect)
    End Sub

    Private Sub drawDiamond(ByVal e As D2DDrawEventArgs, ByVal rect As RectangleF)
        Dim path As New PathGeometry(e.Factory)
        Dim sink As GeometrySink = path.Open
        Dim x As Single = rect.Height / 2.0F
        Dim y As Single = rect.Height / 2.0F
        sink.BeginFigure(New PointF(rect.X, rect.Y + y), FigureBegin.Hollow)
        Dim points(4) As PointF
        points(0) = New PointF(rect.X + x, rect.Y)
        points(1) = New PointF(rect.X + rect.Width - x, rect.Y)
        points(2) = New PointF(rect.X + rect.Width, rect.Y + y)
        points(3) = New PointF(rect.X + rect.Width - x, rect.Y + rect.Height)
        points(4) = New PointF(rect.X + x, rect.Y + rect.Height)
        sink.AddLines(points)
        sink.EndFigure(FigureEnd.Closed)
        sink.Close()
        e.Target.DrawGeometry(path, ColorManager.GetBrush("defaultWhite"), 2 * Game.getSizeFactor)
        sink.Dispose()
        path.Dispose()
    End Sub

    Private Sub fillDiamond(ByVal e As D2DDrawEventArgs, ByVal rect As RectangleF)
        Dim path As New PathGeometry(e.Factory)
        Dim sink As GeometrySink = path.Open
        Dim x As Single = rect.Height / 2.0F
        Dim y As Single = rect.Height / 2.0F
        sink.BeginFigure(New PointF(rect.X, rect.Y + y), FigureBegin.Filled)
        Dim points(4) As PointF
        points(0) = New PointF(rect.X + x, rect.Y)
        points(1) = New PointF(rect.X + rect.Width - x, rect.Y)
        points(2) = New PointF(rect.X + rect.Width, rect.Y + y)
        points(3) = New PointF(rect.X + rect.Width - x, rect.Y + rect.Height)
        points(4) = New PointF(rect.X + x, rect.Y + rect.Height)
        sink.AddLines(points)
        sink.EndFigure(FigureEnd.Closed)
        sink.Close()
        e.Target.FillGeometry(path, ColorManager.GetBrush("defaultWhite"))
        sink.Dispose()
        path.Dispose()
    End Sub

    ''' <summary>
    ''' Erstellt eine neue horizontale Scollbar.
    ''' </summary>
    ''' <remarks></remarks>
    Sub New()
        m_MinValue = 0
        m_MaxValue = 100
        m_Value = 0
        sound1 = New SoundEffect("bt01.wma")
        sound2 = New SoundEffect("bt02.wma")
    End Sub

    Private Sub D2DTrackBar_MouseClick(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles Me.MouseClick
        Dim rWidth As Integer = Convert.ToInt32(Me.Width - (5.0F * Game.getSizeFactor))
        Dim AvailablePixel As Integer = rWidth - trackWidth - Convert.ToInt32(10.0F * Game.getSizeFactor)
        m_Pixel = MathHelper.Clamp(e.X - Convert.ToInt32(trackWidth / 2) - Convert.ToInt32(5.0F * Game.getSizeFactor), 0, AvailablePixel)
        m_Value = MathHelper.Clamp(CInt(GetProzentwert(m_MaxValue - m_MinValue, GetProzentsatz(AvailablePixel, m_Pixel))) + m_MinValue, m_MinValue, m_MaxValue)
        RaiseEvent Scroll(Me, New EventArgs)
        RaiseEvent ValueChanged(Me, New EventArgs)
        If (DateTime.Now - m_MouseDownTime).Ticks < 3000000 AndAlso Not m_Moved Then
            sound1.play()
            m_Moved = False
        End If
    End Sub

    Private Sub m_Parent_MouseMove(ByVal sender As Object, ByVal e As System.Windows.Forms.MouseEventArgs) Handles m_Parent.MouseMove
        If Not m_MouseIsDown Then Exit Sub
        Dim rWidth As Integer = Convert.ToInt32(Me.Width - (5.0F * Game.getSizeFactor))
        Dim AvailablePixel As Integer = rWidth - trackWidth - Convert.ToInt32(10.0F * Game.getSizeFactor)
        m_Pixel = MathHelper.Clamp(e.X - Me.X - Convert.ToInt32(trackWidth / 2) - Convert.ToInt32(5.0F * Game.getSizeFactor), 0, AvailablePixel)
        m_Value = MathHelper.Clamp(CInt(GetProzentwert(m_MaxValue - m_MinValue, GetProzentsatz(AvailablePixel, m_Pixel))) + m_MinValue, m_MinValue, m_MaxValue)
        RaiseEvent Scroll(Me, New EventArgs)
        RaiseEvent ValueChanged(Me, New EventArgs)
        m_Moved = True
    End Sub

    Private Sub TDTrackBar_MouseDown(sender As Object, e As MouseEventArgs) Handles Me.MouseDown
        sound2.play()
        m_MouseDownTime = DateTime.Now
        m_Moved = False
    End Sub

    Private Sub TDTrackBar_MouseUp(sender As Object, e As MouseEventArgs) Handles Me.MouseUp
        sound2.play()
    End Sub

    Private Sub D2DTrackBar_Resize(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Resize
        SetPixel()
    End Sub
End Class
